#include <stdio.h>
#include "types.h"

#define INFOSIZE 32

UBYTE info[INFOSIZE];

int GetINT(int pos)
{
    int val;

    val = (((info[pos + 2] << 8) + info[pos + 1]) << 8) + info[pos];
    if (val >= 0x80000)
	val -= 0x100000;
    return (val);
}

char *MakeCoords(int pos)
{
    static char buffer[8];

    if (pos) {
	buffer[0] = (pos & 15) + 'a' - 1;
	buffer[1] = (pos >> 4) + '0' - 1;
	buffer[2] = 0;
	return (buffer);
    } else {
	return ("--");
    }
}

int main(int argc, char *argv[])
{
    FILE *file;
    UBYTE buffer[16];
    int ply, ext1, ext2;

    if (argc != 2) {
	printf("Usage: PrintInfo filename\n");
	return 0;
    }
    if ((file = fopen(argv[1], "r")) == 0) {
	printf("Cannot open file '%s'\n", argv[1]);
	return 0;
    }
    if (fread(buffer, 1, 13, file) != 13) {
	printf("Cannot read HP header\n");
	fclose(file);
	return 0;
    }
    if (fread(info, 1, INFOSIZE, file) != INFOSIZE) {
	printf("Cannot read info data\n");
	fclose(file);
	return 0;
    }
    printf("VKNG,SKNG          = %s,", MakeCoords(info[0]));
    printf("%s\n", MakeCoords(info[1]));
    printf("VMTL,SMTL          = %d,%d\n", GetINT(2), GetINT(5));
    printf("PLY,DEPTH,EXTDEPTH = %d,%d,%d\n", info[8], info[9], info[10]);
    printf("COL,EPSNT          = %d,%s\n", info[11], MakeCoords(info[12]));
    printf("HI,GOAL,LEGAL      = %d,%d,%d\n", GetINT(13), GetINT(16), info[19]);
    printf("GLSCORE            = %d\n", GetINT(20));
    printf("TOA1,TOH1          = %d,%d\n", info[23], info[24]);
    printf("TOA8,TOH8          = %d,%d\n", info[25], info[26]);
    printf("FLAGS,MOVENUM      = %d,%d\n", info[27], info[28]);
    printf("G_DEPTH,G_EXTDEPTH = %d,%d\n", info[29], info[30]);
    printf("G_EPSNT            = %s\n", MakeCoords(info[31]));

    for (ply = 1; fread(buffer, 1, 4, file) == 4; ply++) {
	ext1 = buffer[2];
	ext2 = buffer[3];
	printf("BEST[%2d]           = %s", ply, MakeCoords(buffer[0]));
	printf("%s %3d ", MakeCoords(buffer[1]), ext1);
	if ((ext1 == 2) || (ext1 == 3)) {
	    printf(" %s\n", MakeCoords(ext2));
	} else {
	    if (ext2 >= 128)
		ext2 -= 256;
	    printf("%3d\n", ext2);
	}

    }
    fclose(file);
    return 0;
}
