LABEL stateval

* StatEval: Compute static score for moves in movelist
*
* Stack: (BRD INFO MOVES -> BRD INFO' MOVES')
*
* Output: HI	    = Best score among the leaf moves in the move list.
*		      HI >= INFTY iff it is possible to capture the enemy king from
*                     this position.
*	  LEGAL     = Number of legal moves. Non-leaf moves are not counted. (They
*                     are potentially illegal). King capture moves are always
*                     considered legal.
*	  best[ply] = The best move (if legal > 0). King captures are not
*                     necessarily saved though.
*	  MOVES'    = The moves that are not leafs in the search tree. Statscore
*                     will have been computed for these moves.
*
* Note: This routine assumes that GOAL < INFTY, so that king capture is
*       always >= GOAL.

ASSEMBLE

PLY_ge_Ext	EQU	1
Opening 	EQU	2
Middle		EQU	3
Ending		EQU	4
tmpflag1	EQU	5
RightFlag	EQU	6

RPL
( :: CK3&Dispatch # 333 )          ( BRD INFO MOVES )

:: ZERO TOTEMPOB CODE
* Input: "BRD" "INFO" "MOVES" #0
	GOSBVL	save_regs
	D1=D1+	15		; D1 -> BRD
	ST=0	WhiteFlag
	LC(5)   COL
	B=C	A
	GOSUB	GetInfoB
	C=C-1	B
	?C#0	B		; if (col != 1) goto Black
	GOYES	+
	ST=1	WhiteFlag
+	ST=1	Opening
	ST=0	Middle
	ST=0	Ending
	B=0	A
	LC(2)   VMTL
	B=C	B
	GOSUB	GetInfoI
	D=C	A
	D0=D0+	(SMTL)-(VMTL)
	C=DAT0	A
	D=D+C	A		; D = VMTL + SMTL = total material value
	LC(5)   7400
	?D>=C	A		; if (D >= 7400) Opening = TRUE;
	GOYES	StageDone
	ST=0	Opening
	ST=1	Middle
	LC(5)   2500
	?D>=C	A		; if (D >= 2500) Middle = TRUE;
	GOYES	StageDone
	ST=0	Middle
	ST=1	Ending		;	    else Ending = TRUE;
StageDone

	LC(2)   LEGAL
	B=C	B
	C=0	A
	D1=D1-	5
	GOSUBL	PutInfoB	; LEGAL = 0
	D1=D1+	5

	LC(2)   PLY
	B=C	B
	GOSUB	GetInfoB
	D0=D0+	(Depth)-(PLY)
	A=DAT0	B
	?C>=A	B
	GOYES	Pass1		; if (ply >= depth) goto Pass1

	D1=D1-	10
	A=DAT1	A
	D0=A
	D0=D0+	5
	C=DAT0	A
	C=C-CON A,5
	D1=D1-	5
	A=DAT1	A
	D1=D1+	15
	D0=A
	D0=D0+	5
	DAT0=C	A		; bint = strlen i nibbles
	GOTO	Pass2

Pass1	ST=0	PLY_ge_Ext
	D0=D0+	(EXTDEPTH)-(Depth)
	A=DAT0	B
	?C<A	B
	GOYES	PLYlExt
	ST=1	PLY_ge_Ext
PLYlExt
	D1=D1-	10
	A=DAT1	A
	D1=D1+	10
	D0=A
	D0=D0+	5
	C=DAT0	A
	C=C-CON A,5		; C = strlen i nibbles
	A=A+CON A,10
	R2=A.F	A		; R2 = &movestring[i] (i=0)
	A=A+C	A
	R1=A.F	A		; R1 = &movestring[nummoves]

MainLp1 A=R1.F	A
	C=R2.F	A
	?C<A	A
	GOYES	InLp1
	GOTO	ExitLp1
InLp1	D0=C
	C=DAT0	8
	R0=C			; R0 = move
	?ST=1	PLY_ge_Ext
	GOYES	EvScore
	CSR	W
	CSR	W
	CSR	A
	CSR	A		; C.B = ext1
	?C=0	B
	GOYES	EvScore 	; if (C == 0) goto EvScore
	C=C-CON B,2
	?C=0	B
	GOYES	EvScore 	; if (C == 2) goto EvScore
	C=C-CON B,2
	?C=0	B
	GOYES	EvScore 	; if (C == 4) goto EvScore
	C=C-1	B
	?C=0	B
	GOYES	EvScore 	; if (C == 5) goto EvScore

SaveIt	D1=D1-	10
	A=DAT1	A		; A = start of movestring
	D1=D1-	5
	C=DAT1	A
	D0=C
	D0=D0+	5
	C=DAT0	A		; C = bint
	A=A+C	A
	C=C+CON A,16
	DAT0=C	A		; bint += 16

	D0=A
	D0=D0+	10
	C=R0
	DAT0=C	8

	D1=D1+	15
	GOTO	NextLp1

******** EvScore ********
*
* EvalStatScore(); legal++
* score = glscore + statscore;
* if (score > hi) {
*   hi = score; best[ply] = move;
*   if (hi >= goal) {
*     moves = ""; return;
*   }
* }

EvScore GOSUB	StatScore
	D=C	A		; D = statscore
	LC(2)   GLSCORE
	B=0	A
	B=C	B
	GOSUB	GetInfoI
	D=D+C	A		; D = statscore + glscore = score

	C=0	A
	D0=(5)  =CRC
	C=DAT0	1
	?CBIT=0 0
	GOYES	ev_add

	CSRB.F	A
	CSRB.F	A
	D=D-C	A		; D -= random value
	GOTO	random_done

ev_add	CSRB.F	A
	CSRB.F	A
	D=D+C	A		; D += random value

random_done

	LC(2)   LEGAL
	B=0	A
	B=C	B
	GOSUB	GetInfoB
	C=C+1	B
	DAT0=C	B		; legal++

	D0=D0-	(LEGAL)-(HI)
	C=DAT0	A
	C=C-D	A		; C = hi - score
	LA(5)   #80000
	?C<A	A
	GOYES	e_notbest
e_best	C=D	A
	DAT0=C	A		; hi = score

	D0=D0-	(HI)-(PLY)
	C=0	A
	C=DAT0	B		; C = PLY
	C=C+C	B
	C=C+C	B
	C=C+C	B
	AD0EX
	A=A+C	A
	LC(2)   (MOVE0)-(PLY)
	A=A+C	A
	D0=A			; D0 = &info[MOVE0+8*PLY]
	C=R0
	DAT0=C	8		; best[ply] = move

	LC(2)   GOAL
	B=C	B
	GOSUB	GetInfoI
	C=C-D	A		; C = goal - score
	?C=0	A
	GOYES	e_goal		; if (hi == goal) goto e_goal
	LA(5)   #80000
	?C<A	A
	GOYES	e_notbest	; if (hi < goal) goto e_notbest

e_goal	D1=D1-	15
	A=DAT1	A
	D1=D1+	15
	D0=A
	D0=D0+	5
	C=0	A
	C=C+1	B
	DAT0=C	A		; bint = 1
	GOTO	Exit
e_notbest

******** End of EvScore ********

NextLp1 A=R2.F	A
	A=A+CON A,16
	R2=A.F	A
	GOTO	MainLp1
ExitLp1

Pass2	D1=D1-	15
	A=DAT1	A
	D0=A
	D0=D0+	5
	C=DAT0	A		; C = strlen i nibbles
	D1=D1+	5
	A=DAT1	A
	D1=D1+	10
	A=A+CON A,10
	R2=A.F	A		; R2 = &movestring[i] (i=0)
	A=A+C	A
	R1=A.F	A		; R1 = &movestring[nummoves]

MainLp2 A=R1.F	A
	C=R2.F	A
	?C>=A	A
	GOYES	ExitLp2
	D0=C
	C=DAT0	8
	R0=C			; R0 = move
	GOSUB	StatScore

	D=C	A
	LC(5)   0-(INFTY)
	C=C+D	A		; C = score - INFTY
	LA(5)   #80000
	?C>=A	A
	GOYES	IsLegal 	; if (score < INFTY) goto IsLegal
	B=0	A
	LC(2)   HI
	B=C	B
	C=D	A
	GOSUBL	PutInfoI	; HI = score
	D0=D0+	(LEGAL)-(HI)
	C=0	A
	C=C+1	B
	DAT0=C	B		; legal = 1
	GOTO	e_goal
IsLegal C=D	A

	A=R2.F	A
	D0=A
	D0=D0+	8
	DAT0=C	A
	A=A+CON A,16
	R2=A.F	A
	GOTO	MainLp2

ExitLp2 D1=D1-	15
	A=DAT1	A
	D1=D1+	15
	D0=A
	D0=D0+	5
	C=DAT0	A		; C = strlen i nibbles
	CSRB.F	A		; convert to bytes
	C=C+1	A		; Compensate for 1_#1-SUB$
	DAT0=C	A		; bint = (bint / 2) + 1

Exit	GOVLNG	rest_loop

********************************************************************
* GetInfoB: Read a BYTE from INFO
*
* Input:  D1  = BRD
*	  B.A = offset (in nibbles)
*
* Output: C.B = info[offset]
*	  D0  = target address
*
* Destroys: A

GetInfoB
	D1=D1-	5
	A=DAT1	A
	A=A+B	A
	D0=A
	C=DAT0	B
	D1=D1+	5
	RTN

********************************************************************
* GetInfoI: Read an INT from INFO
*
* Input:  D1  = BRD
*	  B.A = offset (in nibbles)
*
* Output: C.A = info[offset]
*	  D0  = target address

GetInfoI
	D1=D1-	5
	D0=A
	A=DAT1	A
	A=A+B	A
	AD0EX
	C=DAT0	A
	D1=D1+	5
	RTN

********************************************************************
* StatScore: Compute static score for a move
*
* Input:  D1   = BRD
*	  R0.8 = move
*
* Output: C.A  = statscore
*
* Destroys: A,B,C,D,D0

StatScore
	A=R0
	ASR	W
	ASR	W
	ASR	W
	ASR	W		; A = [ext2 ext1]
	C=0	A
	?A#C	B
	GOYES	e_not0
	GOTO	MtlDone

e_not0	C=C+1	A
	?A#C	B
	GOYES	e_not1
	ASR	A
	ASR	A
	GOSUBL	Gtml
	GOTO	MtlDone

e_not1	C=C+1	A
	?A#C	B
	GOYES	e_not2
	C=0	A
	GOTO	MtlDone

e_not2	C=C+1	A
	?A#C	B
	GOYES	e_not3
	A=0	A
	A=A+1	A		; A = 1 = bonde
	GOSUBL	Gtml
	GOTO	MtlDone

e_not3	C=C+1	A
	?A#C	B
	GOYES	e_not4
	GOTO	rok

e_not4	C=C+1	A
	?A#C	B
	GOYES	e_not5
rok	C=0	A
	?ST=0	Opening
	GOYES	NoOp_1
	LC(2)   10+40           ; 10 bonus, costs 40 to move king (compensation)
NoOp_1	?ST=0	Middle
	GOYES	NoMid_1
	LC(2)   5+10            ; 5 bonus, costs 10 to move king (compensation)
NoMid_1 GOTO	MtlDone

e_not5	A=A-CON B,16		; Promotion piece = ext1-16
	GOSUBL	Gtml
	D=C	A		; D = value(ext1-16)
	ASR	A
	ASR	A
	GOSUBL	Gtml
	D=D+C	A		; D = value(ext1-16) + value(ext2)
	A=0	A
	A=A+1	A
	GOSUBL	Gtml
	D=D-C	A		; C = value(ext1-16) + value(ext2) - value(pawn)
	C=D	A

* Find out piece type of moved piece

MtlDone D=C	A		; D = statscore (used until end of subroutine)
	A=R0
	B=0	A
	B=A	B
	GOSUB	GetBrdB

	?ST=1	WhiteFlag
	GOYES	Wh_1
	C=-C	B
Wh_1	C=C-1	B
	?C#0	B
	GOYES	Not_1
	GOTO	Bonde
Not_1	C=C-1	B
	?C#0	B
	GOYES	Not_2
	GOTO	Hast
Not_2	C=C-1	B
	?C#0	B
	GOYES	Not_3
	GOTO	Lopare
Not_3	C=C-1	B
	?C#0	B
	GOYES	Not_4
	GOTO	Torn
Not_4	C=C-1	B
	?C#0	B
	GOYES	Not_5
	GOTO	Dam
Not_5	GOTO	Kung

************************ Pawn ************************

Bonde	LAaddr	VPawnTable
	?ST=1	WhiteFlag
	GOYES	Wh_3
	LC(5)   (SPawnTable)-(VPawnTable)
	A=A+C	A
Wh_3	C=R0.F	B
	B=0	A
	B=C	B
	GOSUB	GetTable
	C=C+C	A
	D=D-C	A		; score -= 2*XPawnTable[from]
	C=R0.F	A
	CSR	A
	CSR	A
	B=C	B
	GOSUB	GetTable
	C=C+C	A
	D=D+C	A		; score += 2*XPawnTable[to]

* --- Check doubled pawns when capturing

	C=R0
	CSR	W
	CSR	W
	B=0	A
	B=C	B		; B = to
	CSR	A
	CSR	A		; C = ext1
	C=C-1	B
	?C=0	B
	GOYES	PawnHit 	; if (capture) goto PawnHit
	LA(2)   2
	?C#A	B		; if (no EP-capture) goto NoDbl
	GOYES	NoDbl
PawnHit ST=1	tmpflag1
	GOSUB	PawnLine
	GONC	NoDbl
	D=D-CON A,12		; -12 points for doubled pawn
NoDbl

* --- Check closeness to own king (only in middle game)

	?ST=0	Middle
	GOYES	NoNearKing
	LC(2)   VKNG
	?ST=1	WhiteFlag
	GOYES	Wh_13
	C=C+CON B,(SKNG)-(VKNG)
Wh_13	B=0	A
	B=C	B
	GOSUB	GetInfoB
	B=C	B		; B = own king position
	A=B	B
	C=R0.F	B
	GOSUB	Dist
	LA(1)   2
	?C>A	P
	GOYES	WasFar
	D=D-CON A,10
WasFar	A=B	B
	C=R0.F	A
	CSR	A
	CSR	A
	GOSUB	Dist
	LA(1)   2
	?C>A	P
	GOYES	NoNearKing
	D=D+CON A,10		; +10 points if <= 2 squares from own king
NoNearKing

	GOTO	StatDone

************************ Knight ************************

Hast	LAaddr	HastTable
	C=R0.F	B
	B=0	A
	B=C	B
	GOSUB	GetTable
	C=C+C	A
	C=C+C	A
	D=D-C	A		; score -= 4*HastTable[from]
	C=R0.F	A
	CSR	A
	CSR	A
	B=C	B
	GOSUB	GetTable
	C=C+C	A
	C=C+C	A
	D=D+C	A		; score += 4*HastTable[to]

* --- Bonus for closeness (2 squares) to opponent pieces


	GOTO	StatDone

************************ Bishop ************************

Lopare	LAaddr	LoparTable
	C=R0.F	B
	B=0	A
	B=C	B
	GOSUB	GetTable
	D=D-C	A		; score -= LoparTable[from]
	C=R0.F	A
	CSR	A
	CSR	A
	B=C	B
	GOSUB	GetTable
	D=D+C	A		; score += LoparTable[to]

* Check mobility and X-ray mobility

	ST=1	RightFlag
	C=R0.F	A
	A=R0.F	B
	CSR	A
	CSR	A
	C=C-A	B
	LA(2)   128
	?C<A	B
	GOYES	LIsPos
	C=-C	B		; C = abs(to - from)
LIsPos	LA(1)   8
	?C<A	P
	GOYES	UpRight
	ST=0	RightFlag
UpRight A=0	A
	LA(2)   17*2
	?ST=1	RightFlag
	GOYES	IsHor_5
	LA(2)   15*2
IsHor_5 B=0	A
	C=R0.F	B
	B=C	B		; B = from
	C=0	A
	GOSUB	Movable 	; C = Movable(from,ParallelDir)
	LA(2)   15*2
	?ST=1	RightFlag
	GOYES	IsHor_6
	LA(2)   17*2
IsHor_6 GOSUB	Movable 	; C += Movable(from,OrtogonalDir)
	B=C	A
	LCaddr	LopDat
	C=C+B	A
	D0=C
	C=0	A
	C=DAT0	P
	D=D-C	A		; D -= LopDat[mobility(from)]

	A=0	A
	LA(2)   17*2
	?ST=1	RightFlag
	GOYES	IsHor_7
	LA(2)   15*2
IsHor_7 B=0	A
	C=R0.F	B
	B=C	B		; B = from
	C=0	A
	GOSUB	Movable 	; C = Movable(from,ParallelDir)
	A=R0.F	A
	ASR	A
	ASR	A
	B=A	B		; B = to
	LA(5)   15*2
	?ST=1	RightFlag
	GOYES	IsHor_8
	LA(2)   17*2
IsHor_8 GOSUB	Movable 	; C += Movable(to,OrtogonalDir)
	B=C	A
	LCaddr	LopDat
	C=C+B	A
	D0=C
	C=0	A
	C=DAT0	P
	D=D+C	A		; D += LopDat[mobility(to)]
	GOTO	StatDone
LopDat	NIBHEX	02468ACDEFFFFFF

************************ Rook ************************

Torn	ST=1	RightFlag
	C=R0.F	A
	A=R0.F	B
	CSR	A
	CSR	A
	C=C-A	B		; C = to - from
	?C#0	P
	GOYES	IsHoriz 	; if (to.x != from.x) goto IsHoriz
	ST=0	RightFlag
IsHoriz A=0	A
	LA(2)   1*2
	?ST=1	RightFlag
	GOYES	IsHor_1
	LA(2)   16*2
IsHor_1 B=0	A
	C=R0.F	B
	B=C	B		; B = from
	C=0	A
	GOSUB	Movable 	; C = Movable(from,ParallelDir)
	LA(2)   16*2
	?ST=1	RightFlag
	GOYES	IsHor_2
	LA(2)   1*2
IsHor_2 GOSUB	Movable 	; C += Movable(from,OrtogonalDir)
	B=C	A
	LCaddr	RookDat
	C=C+B	A
	D0=C
	C=0	A
	C=DAT0	P
	D=D-C	A		; D -= RookDat[mobility(from)]

	A=0	A
	LA(2)   1*2
	?ST=1	RightFlag
	GOYES	IsHor_3
	LA(2)   16*2
IsHor_3 B=0	A
	C=R0.F	B
	B=C	B		; B = from
	C=0	A
	GOSUB	Movable 	; C = Movable(from,ParallelDir)
	A=R0.F	A
	ASR	A
	ASR	A
	B=A	B		; B = to
	LA(5)   16*2
	?ST=1	RightFlag
	GOYES	IsHor_4
	LA(2)   1*2
IsHor_4 GOSUB	Movable 	; C += Movable(to,OrtogonalDir)
	B=C	A
	LCaddr	RookDat
	C=C+B	A
	D0=C
	C=0	A
	C=DAT0	P
	D=D+C	A		; D += RookDat[mobility(to)]

* --- Check opened and half opened files

	?ST=1	Ending
	GOYES	NoOpLineChk	; if (Ending) { No bonus for opened files }
	C=R0.F	B
	B=0	A
	B=C	B		; B = from
	ST=1	tmpflag1
	GOSUB	PawnLine
	GOC	NoOpen1
	D=D-CON A,10		; Half open file
	ST=0	tmpflag1
	GOSUB	PawnLine
	GOC	NoOpen1
	D=D-CON A,4		; Open file
NoOpen1 C=R0.F	A
	CSR	A
	CSR	A
	B=C	B		; B = to
	ST=1	tmpflag1
	GOSUB	PawnLine
	GOC	NoOpLineChk
	D=D+CON A,10		; Half open file
	ST=0	tmpflag1
	GOSUB	PawnLine
	GOC	NoOpLineChk
	D=D+CON A,4		; Open file
NoOpLineChk

	GOTO	Dam		; Check closeness to enemy king
RookDat NIBHEX	02468ABCDEEEEEE

************************ Queen ************************

Dam	?ST=1	Opening
	GOYES	DamQuit 	; if (opening) goto DamQuit
	B=0	A
	LC(2)   SKNG
	?ST=1	WhiteFlag
	GOYES	Wh_2
	LC(2)   VKNG
Wh_2	B=C	B
	GOSUB	GetInfoB
	B=C	B		; B = EnemyKing
	A=R0.F	B
	GOSUB	TaxiDist	; C = TaxiDist(EnemyKing,from)
	D=D+C	A
	A=R0.F	A
	ASR	A
	ASR	A
	C=B	B
	GOSUB	TaxiDist	; C = TaxiDist(EnemyKing,to)
	D=D-C	A
DamQuit GOTO	StatDone

************************ King ************************

Kung	LC(2)   SMTL
	B=0	A
	B=C	B
	?ST=1	WhiteFlag
	GOYES	Wh_5
	B=B-CON B,(SMTL)-(VMTL)
Wh_5	GOSUB	GetInfoI	; C = value of opponent pieces
	?C#0	A
	GOYES	NoLoneK
	GOTO	StatDone

NoLoneK LAaddr	KingTable_1
	?ST=0	Ending
	GOYES	NoEnd_1
	LC(5)   (KingTable_2)-(KingTable_1)
	A=A+C	A
NoEnd_1 C=R0.F	B
	B=0	A
	B=C	B
	GOSUB	GetTable
	?ST=0	Ending
	GOYES	NoEnd_2
	D=D-C	A
NoEnd_2 C=C+C	A
	D=D-C	A		; score -= x*KingTable_y[from]
	C=R0.F	A
	CSR	A
	CSR	A
	B=C	B
	GOSUB	GetTable
	?ST=0	Ending
	GOYES	NoEnd_3
	D=D+C	A
NoEnd_3 C=C+C	A
	D=D+C	A		; score += x*KingTable_y[to]

* Penalty for moving king before castling (-40 or -10)

	?ST=0	Ending
	GOYES	NoEnd_4
	GOTO	StatDone	; if (Ending) goto StatDone
NoEnd_4 A=R0.F	B
	LC(2)   #25
	?ST=1	WhiteFlag
	GOYES	Wh_6
	LC(2)   #95
Wh_6	?A=C	B
	GOYES	KStaPos
	GOTO	StatDone	; No penalty if king has already moved
KStaPos B=0	A
	LC(2)   TOA1
	?ST=1	WhiteFlag
	GOYES	Wh_4
	LC(2)   TOA8
Wh_4	B=C	B
	GOSUB	GetInfoB
	?C#0	B
	GOYES	NoLong		; if (TOAx > 0) goto NoLong
	LC(2)   #21
	?ST=1	WhiteFlag
	GOYES	Wh_7
	LC(2)   #91
Wh_7	B=C	B
	GOSUBL	GetBrdB
	?ST=1	WhiteFlag
	GOYES	Wh_8
	C=-C	B
Wh_8	LA(2)   4
	?C=A	B
	GOYES	KingPen 	; if (TOAx == 0) && (Brd[ax] == ROOK) goto KingPen

NoLong	LC(2)   TOH1
	?ST=1	WhiteFlag
	GOYES	Wh_9
	LC(2)   TOH8
Wh_9	B=C	B
	GOSUB	GetInfoB
	?C#0	B
	GOYES	StatDone	; if (TOHx > 0) goto StatDone
	LC(2)   #28
	?ST=1	WhiteFlag
	GOYES	Wh_10
	LC(2)   #98
Wh_10	B=C	B
	GOSUBL	GetBrdB
	?ST=1	WhiteFlag
	GOYES	Wh_11
	C=-C	B
Wh_11	LA(2)   4
	?C#A	B
	GOYES	StatDone	; if (TOHx == 0) && (Brd[hx] == ROOK) goto KingPen

KingPen C=0	A
	LC(1)   10
	?ST=1	Middle
	GOYES	Mid_1
	LC(2)   40
Mid_1	D=D-C	A		; score -= penalty

StatDone
	C=D	A
	RTN

********************************************************************
* TaxiDist: Compute taxicab distance between two squares
*
* Input:  D1  = BRD
*	  A.B = sq1
*	  C.B = sq2
*
* Output: C.A = dist
*
* Destroys: A.B

TaxiDist
	?A>=C	B
	GOYES	Tskip1
	ACEX	B
Tskip1	?A>=C	P
	GOYES	Tskip2
	ACEX	P
Tskip2	A=A-C	B
	C=0	A
	C=A	P
	ASR	B
	C=C+A	P
	RTN

********************************************************************
* Dist: Compute distance between two squares
*
* Input:  D1  = BRD
*	  A.B = pos1
*	  C.B = pos2
*
* Output: C.A = dist = max(pos1.x-pos2.x,pos1.y-pos2.y)
*
* Destroys: A.B

Dist	?A>=C	B
	GOYES	Dskip1
	ACEX	B
Dskip1	?A>=C	P
	GOYES	Dskip2
	ACEX	P
Dskip2	A=A-C	B
	C=0	A
	C=A	P
	ASR	B
	?C>=A	P
	RTNYES
	C=A	P
	RTN

********************************************************************
* Movable: Compute rook/bishop mobility
*
* Input:  D1   = BRD
*	  A.A  = direction (1*2,15*2,16*2 or 17*2)
*	  B.A  = pos
*	  C.A  = initial mobility
*
* Output: C.A  += mobility
*
* Destroys: D0

Movable CDEX	A
	RSTK=C

	ST=0	tmpflag1
MvbMLp	GOSUBL	GetBrdB 	; D0 = &BRD[pos]
	D=D-1	A
MvbLp1	CD0EX			; do {
	C=C+A	A
	D0=C			;  D0 += direction
	D=D+1	A		;  D++
	C=DAT0	B
	?C=0	B
	GOYES	MvbLp1		; } while (C == empty)

* Check if piece is a pawn or outside board

	C=C+1	B
	GOC	MvbNDir 	; if (C == -1) goto MvbNDir
	C=C-CON B,2
	?C=0	B
	GOYES	MvbNDir 	; if (C == 1) goto MvbNDir
	C=C-CON B,6
	?C=0	B
	GOYES	MvbNDir 	; if (C == 7) goto MvbNDir

* X-ray mobility (through 1 piece, but not pawns)

	D=D-1	A
MvbLp2	CD0EX			; do {
	C=C+A	A
	D0=C			;  D0 += direction
	D=D+1	A		;  D++
	C=DAT0	B
	?C=0	B
	GOYES	MvbLp2		; } while (C == empty)

MvbNDir A=-A	A		; dir = -dir
	?ST=1	tmpflag1
	GOYES	MvbDone
	ST=1	tmpflag1
	GOTO	MvbMLp		; Execute loop twice

MvbDone C=RSTK
	CDEX	A
	RTN

********************************************************************
* PawnLine: Check for pawns of a certain color on a file
*
* Input:  D1  = BRD
*	  B.A = pos (any square on the file to check)
*	  ST.tmpflag1 = TRUE if checking own pawns
*
* Output: Carry = TRUE if pawns found, FALSE otherwise
*
* Destroys: A,C,D0
*	   B.A	= any square on the file (ie this routine can be called
*                 again without changing B)

PawnLine
	C=B	B
	P=	1
	LC(1)   2
	P=	0
	B=C	B
	GOSUBL	GetBrdB 	; D0 = &BRD[x1]
	LA(2)   1
	?ST=1	WhiteFlag
	GOYES	Wh_12
	A=-A	B
Wh_12	?ST=1	tmpflag1
	GOYES	friend
	A=-A	B
friend	P=	16-8		; Kolla 8 rutor
PLineLp C=DAT0	B
	?C=A	B
	GOYES	RetTRUE 	; if (piece == correct pawn) goto RetTRUE
	D0=D0+	16
	D0=D0+	16
	P=P+1
	GONC	PLineLp
	RTNCC			; return FALSE
RetTRUE P=	0
	RTNSC			; return TRUE

********************************************************************
* GetTable: Read a value from an array
*
* Input:  D1  = BRD
*	  A.A = table
*	  B.A = pos
*
* Output: C.A = value
*
* Destroys: D0

GetTable
	D0=A			; D0 = &table[0]
	C=0	A
	C=B	P
	C=C-1	P
	A=A+C	A
	C=B	B
	CSR	B
	C=C-CON B,2
	C=C+C	B
	C=C+C	B
	C=C+C	B
	A=A+C	A
	AD0EX			; A = &table[0]; D0 = &table[pos]
	C=0	A
	C=DAT0	P
	RTN

HastTable
	NIBHEX	01233210	; Multiply by 4
	NIBHEX	12455421
	NIBHEX	24677642
	NIBHEX	35788753
	NIBHEX	35788753
	NIBHEX	24677642
	NIBHEX	12455421
	NIBHEX	01233210

LoparTable
	NIBHEX	00000000
	NIBHEX	08444480
	NIBHEX	04888840
	NIBHEX	04888840
	NIBHEX	04888840
	NIBHEX	04888840
	NIBHEX	08444480
	NIBHEX	00000000

VPawnTable
	NIBHEX	00000000	; Multiply by 2
	NIBHEX	22200222
	NIBHEX	34155143
	NIBHEX	34688643
	NIBHEX	468CC864
	NIBHEX	57AEEA75
SPawnTable
	NIBHEX	68CFFC86
	NIBHEX	68CFFC86

	NIBHEX	57AEEA75
	NIBHEX	468CC864
	NIBHEX	34688643
	NIBHEX	34155143
	NIBHEX	22200222
	NIBHEX	00000000

KingTable_1
	NIBHEX	CCA77ACC	; Multiply by 2
	NIBHEX	AA8668AA
	NIBHEX	64222246
	NIBHEX	42000024
	NIBHEX	42000024
	NIBHEX	64222246
	NIBHEX	AA8668AA
	NIBHEX	CCA77ACC

KingTable_2
	NIBHEX	02466420	; Multiply by 3
	NIBHEX	24688642
	NIBHEX	468AA864
	NIBHEX	68ACCA86
	NIBHEX	68ACCA86
	NIBHEX	468AA864
	NIBHEX	24688642
	NIBHEX	02466420

ENDCODE
  1_#1-SUB$
;
